@@ -67,20 +67,21 @@ Fit_Index <- function(ind_slot='Ind', indcv_slot="CV_Ind", Data_out,
   # Calculate statistics
   Stats_List <- lapply(1:nsim, function(x) Calc_Stats(lResids_Hist[x,]))
   Stats <- do.call('rbind', Stats_List)
+  check_Index <-check_Index_Fit(Stats, ind_slot)
   
-  # Generate residuals for projections
-  Resid_Hist <- exp(lResids_Hist) # historical residuals in normal space
-  Resid_Proj <- Gen_Residuals(Stats, nsim, proyears)
-  
-  ObsPars[[obs_err_var]][, 1:nyears] <- Resid_Hist # update Obs Error
-  ObsPars[[obs_err_var]][, (nyears+1):(nyears+proyears)] <- Resid_Proj
-  
-  stat_var <- switch(ind_slot,
-                     Ind = 'Ind_Stat',
-                     SpInd = 'SpInd_Stat',
-                     VInd = 'VInd_Stat')
-  
-  
+  if (check_Index) {
+    # Generate residuals for projections
+    Resid_Hist <- exp(lResids_Hist) # historical residuals in normal space
+    Resid_Proj <- Gen_Residuals(Stats, nsim, proyears)
+    
+    ObsPars[[obs_err_var]][, 1:nyears] <- Resid_Hist # update Obs Error
+    ObsPars[[obs_err_var]][, (nyears+1):(nyears+proyears)] <- Resid_Proj
+    
+    stat_var <- switch(ind_slot,
+                       Ind = 'Ind_Stat',
+                       SpInd = 'SpInd_Stat',
+                       VInd = 'VInd_Stat')
+  }
   ObsPars[[stat_var]] <- Stats[,1:2]
   list(Data_out=Data_out, ObsPars=ObsPars)
   
@@ -94,18 +95,18 @@ Calc_Residuals <- function(sim.index, obs.ind, beta=NA) {
     stop('Observed index cannot have negative values', call.=FALSE)
   
   # standardize index and biomass to mean 1 
-  s.sim.index <- sim.index/mean(sim.index, na.rm=TRUE)
   s.obs.ind <- obs.ind/mean(obs.ind, na.rm=TRUE)
+  notnas <- !is.na(s.obs.ind)
+  s.sim.index <- sim.index/mean(sim.index[notnas], na.rm=TRUE)
   
   # convert to log space
   l.sim.index <- log(s.sim.index)
   l.obs.index <- log(s.obs.ind)
   
   # mean 0 
-  l.sim.index <- l.sim.index-mean(l.sim.index, na.rm = TRUE)
+  l.sim.index <- l.sim.index-mean(l.sim.index[notnas], na.rm = TRUE)
   l.obs.index <- l.obs.index-mean(l.obs.index, na.rm = TRUE)
-  
-  
+
   # estimate beta (if not provided)
   if (is.na(beta)) {
     opt<-optimize(getbeta,x=exp(l.sim.index),y=exp(l.obs.index),
