@@ -68,7 +68,7 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), dir=NULL, overwrite=F
     OMexample(dir)
     return(message("Creating Example Files in ", dir))
   }
-  if (class(name) != 'character') stop("name must be text", call.=FALSE)
+  if (!methods::is(name, 'character')) stop("name must be text", call.=FALSE)
 
   ## Create Folders ####
   if(!dir.exists(file.path(dir,'data'))) dir.create(file.path(dir,'data'))
@@ -228,137 +228,137 @@ OMinit <- function(name=NULL, ..., files=c('xlsx', 'rmd'), dir=NULL, overwrite=F
 }
 
 
-#' Load OM from Excel file
-#'
-#' Imports an OM from a correctly formatted Excel file. Create the Excel spreadsheet template
-#' using `OMinit` and document each slot in the corresponding text file.
-#'
-#' An error message will alert if any slots are missing values, or if the Excel file is missing
-#' the required tabs.
-#'
-#' @param name Name of the OM Excel file. Provide full file path if not in current directory.
-#' @param cpars An optional list of custom parameters (single parameters are a vector nsim
-#' long, time series are a matrix nsim rows by nyears columns)
-#' @param msg Should messages be printed?
-#'
-#' @return An object of class OM
-#' @export
-#' @author A. Hordyk
-#'
-#' @examples
-#' \dontrun{
-#' OMinit('myOM', templates=list(Stock='Herring', Fleet='Generic_Fleet', Obs='Generic_Obs',
-#' Imp='Perfect_Imp'), overwrite=TRUE)
-#' myOM <- XL2OM('myOM.xlsx')
-#'
-#' }
-XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
-  if (!requireNamespace("readxl", quietly = TRUE)) {
-    stop("Package \"readxl\" needed for this function to work. Please install it.",
-         call. = FALSE)
-  }
-  # Load the Excel File ####
-  if (is.null(name)) {
-    fls <- list.files(pattern=".xlsx", ignore.case = TRUE)
-    fls <- fls[!grepl('~', fls)]
-    if (length(fls) == 0) stop('Name not provided and no .xlsx files found.', call.=FALSE)
-    if (length(fls) > 1) stop("Name not provided and multiple .xlsx files found", call.=FALSE)
-    name <- fls
-  }
-
-  if (class(name) != 'character') stop("file name must be provided", call.=FALSE)
-
-  if (nchar(tools::file_ext(name)) == 0) {
-    xl.fname1 <- paste0(name, ".xlsx")
-    xl.fname2 <- paste0(name, ".xls")
-    fls <- file.exists(c(xl.fname1, xl.fname2))
-    if (sum(fls) == 0) stop(xl.fname1, " or ", xl.fname2, " not found")
-    if (sum(fls) > 1) stop(name, " found with multiple extensions. Specify file extension.", call.=FALSE)
-    name <- c(xl.fname1, xl.fname2)[fls]
-  }
-  if (!file.exists(name)) stop(name, " not found", call.=FALSE)
-  message("Reading ", name)
-  sheetnames <- readxl::excel_sheets(name)  # names of the sheets
-  reqnames <- c("OM", "Stock", "Fleet", "Obs", "Imp")
-  ind <- which(!reqnames%in% sheetnames)
-  if (length(ind)>0) stop("Sheets: ", paste(reqnames[ind], ""), "not found in ", name, call.=FALSE)
-
-  count <- 1
-  tempObj <- vector("list", 4)
-  for (obj in c("Stock", "Fleet", "Obs", "Imp")) {
-    sht <- suppressMessages(as.data.frame(readxl::read_excel(name, sheet = obj, col_names = FALSE)))
-    rows <- sht[,1]
-    rows <- rows[!rows == "Slot"]
-    rows <- rows[!is.na(rows)]
-    ind <- which(!rows %in% slotNames(obj))
-    if (length(ind)>0) {
-      warning(paste(rows[ind], ""), "are not valid slots in object class ", obj)
-    }
-
-    if (all(dim(sht) == 0)) stop("Nothing found in sheet: ", obj, call.=FALSE)
-    tmpfile <- tempfile(fileext=".csv")
-    writeCSV2(inobj = sht, tmpfile, objtype = obj)
-    if (ncol(sht)<2) {
-      unlink(tmpfile)
-      warning("No parameter values found in Sheet: ", obj, ". Using defaults", call.=FALSE)
-      tempObj[[count]] <- new(obj)
-    } else {
-      tempObj[[count]] <- new(obj, tmpfile)
-    }
-    unlink(tmpfile)
-    count <- count + 1
-  }
-
-  # Operating Model
-  OM <- new("OM", Stock = tempObj[[1]], Fleet = tempObj[[2]],
-            Obs = tempObj[[3]], Imp=tempObj[[4]])
-
-  # Read in the OM sheet
-  sht <- suppressMessages(as.data.frame(readxl::read_excel(name, sheet = "OM", col_names = FALSE)))
-  dat <- sht # sht[,1:2]
-  dat <- dat[which(dat[,1] != "Slot"),]
-  # if (ncol(sht)>2) warning("More than two columns found in Sheet OM. Values in columns C+ are ignored")
-  if (ncol(sht)<2) {
-    message("No values found for OM slots in Sheet OM. Using defaults")
-  } else {
-    for (xx in 1:nrow(dat)) {
-      val <- dat[xx, 2:ncol(dat)]
-      if (length(val)) {
-        if (!dat[xx,1] %in% c("Name", "Agency", "Region", "Sponsor")) {
-          options(warn=-1)
-          val <- as.numeric(val)
-          options(warn=1)
-          val <- val[!is.na(val)]
-          if (.hasSlot(OM, dat[xx,1])) slot(OM, dat[xx, 1]) <- val
-        } else  {
-          val <- val[!is.na(val)]
-          if (.hasSlot(OM, dat[xx,1])) slot(OM, dat[xx, 1]) <- val
-        }
-
-      } else{
-        message("No value found for OM slot ", dat[xx,1], ". Using default: ", slot(OM, dat[xx, 1]))
-      }
-    }
-  }
-
-  if (!is.null(cpars)) {
-    if (class(cpars) == "list") {
-      OM@cpars <- cpars
-    } else {
-      stop("'cpars' must be a list", call.=FALSE)
-    }
-  }
-  # tt <- ChkObj(OM, FALSE)
-  if (msg) {
-    message('OM successfully imported\n')
-    message("Document OM slots in .rmd file (probably ", tools::file_path_sans_ext(name), ".rmd),
-  and run 'OMdoc' if OM parameter values have changed." )
-  }
-
-  OM
-}
-
-
+# #' Load OM from Excel file
+# #' 
+# #' Imports an OM from a correctly formatted Excel file. Create the Excel spreadsheet template
+# #' using `OMinit` and document each slot in the corresponding text file.
+# #' 
+# #' An error message will alert if any slots are missing values, or if the Excel file is missing
+# #' the required tabs.
+# #' 
+# #' @param name Name of the OM Excel file. Provide full file path if not in current directory.
+# #' @param cpars An optional list of custom parameters (single parameters are a vector nsim
+# #' long, time series are a matrix nsim rows by nyears columns)
+# #' @param msg Should messages be printed?
+# #' 
+# #' @return An object of class OM
+# #' @export
+# #' @author A. Hordyk
+# #' 
+# #' @examples
+# #' \dontrun{
+# #' OMinit('myOM', templates=list(Stock='Herring', Fleet='Generic_Fleet', Obs='Generic_Obs',
+# #' Imp='Perfect_Imp'), overwrite=TRUE)
+# #' myOM <- XL2OM('myOM.xlsx')
+# #' 
+# #' }
+# XL2OM <- function(name=NULL, cpars=NULL, msg=TRUE) {
+#   if (!requireNamespace("readxl", quietly = TRUE)) {
+#     stop("Package \"readxl\" needed for this function to work. Please install it.",
+#          call. = FALSE)
+#   }
+#   # Load the Excel File ####
+#   if (is.null(name)) {
+#     fls <- list.files(pattern=".xlsx", ignore.case = TRUE)
+#     fls <- fls[!grepl('~', fls)]
+#     if (length(fls) == 0) stop('Name not provided and no .xlsx files found.', call.=FALSE)
+#     if (length(fls) > 1) stop("Name not provided and multiple .xlsx files found", call.=FALSE)
+#     name <- fls
+#   }
+# 
+#   if (!methods::is(name, 'character')) stop("file name must be provided", call.=FALSE)
+# 
+#   if (nchar(tools::file_ext(name)) == 0) {
+#     xl.fname1 <- paste0(name, ".xlsx")
+#     xl.fname2 <- paste0(name, ".xls")
+#     fls <- file.exists(c(xl.fname1, xl.fname2))
+#     if (sum(fls) == 0) stop(xl.fname1, " or ", xl.fname2, " not found")
+#     if (sum(fls) > 1) stop(name, " found with multiple extensions. Specify file extension.", call.=FALSE)
+#     name <- c(xl.fname1, xl.fname2)[fls]
+#   }
+#   if (!file.exists(name)) stop(name, " not found", call.=FALSE)
+#   message("Reading ", name)
+#   sheetnames <- readxl::excel_sheets(name)  # names of the sheets
+#   reqnames <- c("OM", "Stock", "Fleet", "Obs", "Imp")
+#   ind <- which(!reqnames%in% sheetnames)
+#   if (length(ind)>0) stop("Sheets: ", paste(reqnames[ind], ""), "not found in ", name, call.=FALSE)
+# 
+#   count <- 1
+#   tempObj <- vector("list", 4)
+#   for (obj in c("Stock", "Fleet", "Obs", "Imp")) {
+#     sht <- suppressMessages(as.data.frame(readxl::read_excel(name, sheet = obj, col_names = FALSE)))
+#     rows <- sht[,1]
+#     rows <- rows[!rows == "Slot"]
+#     rows <- rows[!is.na(rows)]
+#     ind <- which(!rows %in% slotNames(obj))
+#     if (length(ind)>0) {
+#       warning(paste(rows[ind], ""), "are not valid slots in object class ", obj)
+#     }
+# 
+#     if (all(dim(sht) == 0)) stop("Nothing found in sheet: ", obj, call.=FALSE)
+#     tmpfile <- tempfile(fileext=".csv")
+#     writeCSV2(inobj = sht, tmpfile, objtype = obj)
+#     if (ncol(sht)<2) {
+#       unlink(tmpfile)
+#       warning("No parameter values found in Sheet: ", obj, ". Using defaults", call.=FALSE)
+#       tempObj[[count]] <- new(obj)
+#     } else {
+#       tempObj[[count]] <- new(obj, tmpfile)
+#     }
+#     unlink(tmpfile)
+#     count <- count + 1
+#   }
+# 
+#   # Operating Model
+#   OM <- new("OM", Stock = tempObj[[1]], Fleet = tempObj[[2]],
+#             Obs = tempObj[[3]], Imp=tempObj[[4]])
+# 
+#   # Read in the OM sheet
+#   sht <- suppressMessages(as.data.frame(readxl::read_excel(name, sheet = "OM", col_names = FALSE)))
+#   dat <- sht # sht[,1:2]
+#   dat <- dat[which(dat[,1] != "Slot"),]
+#   # if (ncol(sht)>2) warning("More than two columns found in Sheet OM. Values in columns C+ are ignored")
+#   if (ncol(sht)<2) {
+#     message("No values found for OM slots in Sheet OM. Using defaults")
+#   } else {
+#     for (xx in 1:nrow(dat)) {
+#       val <- dat[xx, 2:ncol(dat)]
+#       if (length(val)) {
+#         if (!dat[xx,1] %in% c("Name", "Agency", "Region", "Sponsor")) {
+#           options(warn=-1)
+#           val <- as.numeric(val)
+#           options(warn=1)
+#           val <- val[!is.na(val)]
+#           if (.hasSlot(OM, dat[xx,1])) slot(OM, dat[xx, 1]) <- val
+#         } else  {
+#           val <- val[!is.na(val)]
+#           if (.hasSlot(OM, dat[xx,1])) slot(OM, dat[xx, 1]) <- val
+#         }
+# 
+#       } else{
+#         message("No value found for OM slot ", dat[xx,1], ". Using default: ", slot(OM, dat[xx, 1]))
+#       }
+#     }
+#   }
+# 
+#   if (!is.null(cpars)) {
+#     if (methods::is(cpars,"list")) {
+#       OM@cpars <- cpars
+#     } else {
+#       stop("'cpars' must be a list", call.=FALSE)
+#     }
+#   }
+#   # tt <- ChkObj(OM, FALSE)
+#   if (msg) {
+#     message('OM successfully imported\n')
+#     message("Document OM slots in .rmd file (probably ", tools::file_path_sans_ext(name), ".rmd),
+#   and run 'OMdoc' if OM parameter values have changed." )
+#   }
+# 
+#   OM
+# }
+# 
+# 
 
 
 
@@ -408,9 +408,9 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
   if (is.null(dir)) dir <- getwd()
   OMXLname <- NULL
 
-  if (class(OM) == "OM") {
+  if (methods::is(OM,"OM")) {
     # nothing
-  } else if (class(OM) == 'character') {
+  } else if (methods::is(OM, 'character')) {
     OMXLname <- OM
     OM <- XL2OM(file.path(dir,OM), msg=FALSE)
   } else if (is.null(OM)) {
@@ -444,8 +444,8 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
       rmd.source <- paste0(rmd.source, ".rmd")
     } else if (tools::file_ext(rmd.source) != "rmd") stop("rmd.source extension must be rmd", call.=FALSE)
 
-    if (!file.exists(rmd.source)) stop(rmd.source, " not found in ", dir, call.=FALSE)
-    message("Reading ", file.path(dir,rmd.source))
+    if (!file.exists(file.path(dir,rmd.source))) stop(rmd.source, " not found in ", dir, call.=FALSE)
+    message_info("Reading ", file.path(dir,rmd.source))
     textIn <- readLines(file.path(dir,rmd.source))
   }
 
@@ -577,7 +577,7 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
       if (chkFile) {
         # OM has been documented before - check if it has changed
         testOM <- readRDS(file.path(dir,paste0("build/", fileName, '.dat')))
-        if (class(testOM) == 'OM') {
+        if (methods::is(testOM,'OM')) {
           # check if OM has changed
           changed <- rep(FALSE, length(slotNames(OM)))
           for (sl in seq_along(slotNames(OM))) {
@@ -586,8 +586,8 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
             newOM <- slot(testOM, slotNames(OM)[sl])
             oldOM <- oldOM[!is.na(oldOM)]
             newOM <- newOM[!is.na(newOM)]
-            if (class(oldOM) !='character') {
-              if (class(oldOM) != 'list') {
+            if (!methods::is(oldOM,'character')) {
+              if (!methods::is(oldOM, 'list')) {
                 if (length(oldOM)<1 || !is.finite(oldOM)) oldOM <- 0
                 if (length(newOM)<1 || !is.finite(newOM)) newOM <- 0
                 if (any(oldOM != newOM)) changed[sl] <- TRUE
@@ -596,7 +596,8 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
                   changed[sl] <- TRUE
                 } else if (length(oldOM)>0){
                   for (xx in 1:length(oldOM)) {
-                    if(any(oldOM[[xx]] != newOM[[xx]]))changed[sl] <- TRUE
+                    if (!inherits(oldOM[[xx]], 'Data'))
+                      if(any(oldOM[[xx]] != newOM[[xx]]))changed[sl] <- TRUE
 
                   }
                 }
@@ -766,6 +767,9 @@ OMdoc <- function(OM=NULL, rmd.source=NULL, overwrite=FALSE, out.file=NULL,
     }
 
     if (length(OM@cpars$Find)>0) {
+      fstYr <- (OM@CurrentYr -  OM@nyears + 1)
+      lstYr <- OM@CurrentYr
+      EffYears <-fstYr:lstYr
       lower <- as.numeric(signif(apply(OM@cpars$Find, 2, min),3))
       upper <- as.numeric(signif(apply(OM@cpars$Find, 2, max),3))
       Effvals <- data.frame(EffYears=EffYears, EffLower=lower, EffUpper=upper)
@@ -1281,7 +1285,7 @@ plotEff <- function(OM, Pars, nsamp=3, col="darkgray",
 
 plotDep <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col="darkgray",
                     breaks=10, lwd=2) {
-  if (class(OM) != "OM") stop("Must supply object of class 'OM'")
+  if (!methods::is(OM,"OM")) stop("Must supply object of class 'OM'")
 
   if (is.finite(OM@nyears)) nyears <- OM@nyears
   if (is.finite(OM@proyears)) proyears <- OM@proyears
@@ -1321,7 +1325,7 @@ plotDep <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col
 
 
 plotMat <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col="darkgray", breaks=10, lwd=2) {
-  if (class(OM) != "OM") stop("Must supply object of class 'OM'")
+  if (!methods::is(OM,"OM")) stop("Must supply object of class 'OM'")
 
   if (is.finite(OM@nyears)) nyears <- OM@nyears
   if (is.finite(OM@proyears)) proyears <- OM@proyears
@@ -1366,7 +1370,7 @@ plotMat <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col
 
 
 plotSelect <- function(OM, Pars=NULL, pyears=4, sim=NA, type="l") {
-  if (class(OM) != "OM") stop("Object must be class 'OM' ")
+  if (!methods::is(OM, "OM")) stop("Object must be class 'OM' ")
   nsim <- OM@nsim
   if (!is.na(sim)) OM@nsim <- nsim <- max(10, sim) # OM@nsim
   years <- OM@nyears + OM@proyears
@@ -1438,7 +1442,7 @@ plotSelect <- function(OM, Pars=NULL, pyears=4, sim=NA, type="l") {
 
 plotGrowth <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col="darkgray",
                        breaks=10, lwd=2) {
-  if (class(OM) != "OM") stop("Must supply object of class 'OM'")
+  if (!methods::is(OM, "OM")) stop("Must supply object of class 'OM'")
 
   if (is.finite(OM@nyears)) nyears <- OM@nyears
   if (is.finite(OM@proyears)) proyears <- OM@proyears
@@ -1553,7 +1557,7 @@ plotGrowth <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3,
 
 plotRec <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3,
                     col="darkgray", breaks=10, lwd=2) {
-  if (class(OM) != "OM") stop("Must supply object of class 'OM'")
+  if (!methods::is(OM, "OM")) stop("Must supply object of class 'OM'")
 
   if (is.finite(OM@nyears)) nyears <- OM@nyears
   if (is.finite(OM@proyears)) proyears <- OM@proyears
@@ -1625,7 +1629,7 @@ plotRec <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3,
 
 # Plot Natural Mortality
 plotM2 <- function(OM, Pars=NULL, nsim=48, nyears=50, proyears=50, nsamp=3, col="darkgray", breaks=10, lwd=2) {
-  if (class(OM) != "OM") stop("Must supply object of class 'OM'")
+  if (!methods::is(OM, "OM")) stop("Must supply object of class 'OM'")
 
   if (is.finite(OM@nyears)) nyears <- OM@nyears
   if (is.finite(OM@proyears)) proyears <- OM@proyears
