@@ -594,13 +594,15 @@ SampleStockPars <- function(Stock, nsim=48, nyears=80, proyears=50, cpars=NULL,
           oksims2 <- which(apply(Mat_age[,,yr], 1, max) < 0.95)
         }
         
-        
-        if (length(oksims)<1 ) {
+        if (length(oksims)<1 | length(oksims2)<1) {
           if (length(oksims)<1) {
             # no maturity-at-age >= 0.95
-            age95array[,yr] <- maxage # set to 1.5 if < 1
+            age95array[,yr] <- maxage #
+          }
+          if (length(oksims2)<1) {
+            # no maturity-at-age <= 0.95
+            age95array[,yr] <- 1.5 # set to 1.5 if < 1
           }
-          
         } else {
           age95array[,yr] <- unlist(sapply(1:nsim, function(x)
             LinInterp(Mat_age[x,, yr], y=0:(n_age-1), 0.95)))
@@ -1640,15 +1642,15 @@ SampleObsPars <- function(Obs, nsim=NULL, cpars=NULL, Stock=NULL,
     cond <- StockPars$hs > 0.6
     hsim[cond] <- 0.2 + rbeta(sum(StockPars$hs > 0.6),
                               alphaconv((StockPars$hs[cond] - 0.2)/0.8,
-                                        (1 - (StockPars$hs[cond] - 0.2)/0.8) * Obs@hbiascv),
+                                        (1 - (StockPars$hs[cond] - 0.2)/0.8) * Obs@hbiascv[1]),
                               betaconv((StockPars$hs[cond] - 0.2)/0.8,
-                                       (1 - (StockPars$hs[cond] - 0.2)/0.8) * Obs@hbiascv)) * 0.8
+                                       (1 - (StockPars$hs[cond] - 0.2)/0.8) * Obs@hbiascv[1])) * 0.8
 
     hsim[!cond] <- 0.2 + rbeta(sum(StockPars$hs <= 0.6),
                                alphaconv((StockPars$hs[!cond] - 0.2)/0.8,
-                                         (StockPars$hs[!cond] - 0.2)/0.8 * Obs@hbiascv),
+                                         (StockPars$hs[!cond] - 0.2)/0.8 * Obs@hbiascv[1]),
                                betaconv((StockPars$hs[!cond] - 0.2)/0.8,
-                                        (StockPars$hs[!cond] - 0.2)/0.8 * Obs@hbiascv)) * 0.8
+                                        (StockPars$hs[!cond] - 0.2)/0.8 * Obs@hbiascv[1])) * 0.8
     hbias <- hsim/StockPars$hs  # back calculate the simulated bias
   } else {
     hbias <- hsim/StockPars$hs  # back calculate the simulated bias
